@@ -4,15 +4,18 @@ use cap_primitives::AmbientAuthority;
 #[cfg(not(windows))]
 use io_extras::os::rustix::{AsRawFd, FromRawFd, IntoRawFd, RawFd};
 #[cfg(not(windows))]
-use io_lifetimes::{AsFd, BorrowedFd, FromFd, IntoFd, OwnedFd};
+use io_lifetimes::{AsFd, BorrowedFd, OwnedFd};
 #[cfg(windows)]
-use io_lifetimes::{AsHandle, BorrowedHandle, FromHandle, IntoHandle, OwnedHandle};
+use io_lifetimes::{AsHandle, BorrowedHandle, OwnedHandle};
 use std::io::{self, IoSlice, IoSliceMut, Read, Seek, SeekFrom, Write};
 use std::path::Path;
 use std::{fmt, fs, process};
 #[cfg(windows)]
 use {
-    io_extras::os::windows::{AsRawHandleOrSocket, IntoRawHandleOrSocket, RawHandleOrSocket},
+    io_extras::os::windows::{
+        AsHandleOrSocket, AsRawHandleOrSocket, BorrowedHandleOrSocket, IntoRawHandleOrSocket,
+        OwnedHandleOrSocket, RawHandleOrSocket,
+    },
     std::os::windows::io::{AsRawHandle, FromRawHandle, IntoRawHandle, RawHandle},
 };
 
@@ -20,9 +23,9 @@ use {
 ///
 /// This corresponds to [`std::fs::File`].
 ///
-/// Note that this `File` has no `open` or `create` methods. To open or create
-/// a file, you must first obtain a [`Dir`] containing the path, and then call
-/// [`Dir::open`] or [`Dir::create`].
+/// This `File` has no `open` or `create` methods. To open or create a file,
+/// first obtain a [`Dir`] containing the path, and then call [`Dir::open`] or
+/// [`Dir::create`].
 ///
 /// [`Dir`]: crate::fs::Dir
 /// [`Dir::open`]: crate::fs::Dir::open
@@ -47,15 +50,6 @@ impl File {
         self.std
     }
 
-    /// Returns a new [`OpenOptions`] object.
-    ///
-    /// This corresponds to [`std::fs::File::with_options`].
-    #[inline]
-    #[cfg(with_options)]
-    pub fn with_options() -> OpenOptions {
-        OpenOptions::new()
-    }
-
     /// Attempts to sync all OS-internal metadata to disk.
     ///
     /// This corresponds to [`std::fs::File::sync_all`].
@@ -123,7 +117,28 @@ impl File {
     ) -> io::Result<Self> {
         let std = open_ambient(
             path.as_ref(),
-            &OpenOptions::new().read(true),
+            OpenOptions::new().read(true),
+            ambient_authority,
+        )?;
+        Ok(Self::from_std(std))
+    }
+
+    /// Constructs a new instance of `Self` in write-only mode by opening,
+    /// creating or truncating, the given path as a file using the host
+    /// process' ambient authority.
+    ///
+    /// # Ambient Authority
+    ///
+    /// This function is not sandboxed and may access any path that the host
+    /// process has access to.
+    #[inline]
+    pub fn create_ambient<P: AsRef<Path>>(
+        path: P,
+        ambient_authority: AmbientAuthority,
+    ) -> io::Result<Self> {
+        let std = open_ambient(
+            path.as_ref(),
+            OpenOptions::new().write(true).create(true).truncate(true),
             ambient_authority,
         )?;
         Ok(Self::from_std(std))
@@ -146,6 +161,15 @@ impl File {
         let std = open_ambient(path.as_ref(), options, ambient_authority)?;
         Ok(Self::from_std(std))
     }
+
+    /// Returns a new `OpenOptions` object.
+    ///
+    /// This corresponds to [`std::fs::File::options`].
+    #[must_use]
+    #[inline]
+    pub fn options() -> OpenOptions {
+        OpenOptions::new()
+    }
 }
 
 #[inline]
@@ -158,6 +182,9 @@ fn permissions_into_std(file: &fs::File, permissions: Permissions) -> io::Result
     permissions.into_std(file)
 }
 
+// Safety: `FilelikeViewType` is implemented for `std::fs::File`.
+unsafe impl io_lifetimes::views::FilelikeViewType for File {}
+
 #[cfg(not(windows))]
 impl FromRawFd for File {
     #[inline]
@@ -167,10 +194,10 @@ impl FromRawFd for File {
 }
 
 #[cfg(not(windows))]
-impl FromFd for File {
+impl From<OwnedFd> for File {
     #[inline]
-    fn from_fd(fd: OwnedFd) -> Self {
-        Self::from_std(fs::File::from_fd(fd))
+    fn from(fd: OwnedFd) -> Self {
+        Self::from_std(fs::File::from(fd))
     }
 }
 
@@ -183,10 +210,10 @@ impl FromRawHandle for File {
 }
 
 #[cfg(windows)]
-impl FromHandle for File {
+impl From<OwnedHandle> for File {
     #[inline]
-    fn from_handle(handle: OwnedHandle) -> Self {
-        Self::from_std(fs::File::from_handle(handle))
+    fn from(handle: OwnedHandle) -> Self {
+        Self::from_std(fs::File::from(handle))
     }
 }
 
@@ -230,6 +257,14 @@ impl AsRawHandleOrSocket for File {
     }
 }
 
+#[cfg(windows)]
+impl AsHandleOrSocket for File {
+    #[inline]
+    fn as_handle_or_socket(&self) -> BorrowedHandleOrSocket<'_> {
+        self.std.as_handle_or_socket()
+    }
+}
+
 #[cfg(not(windows))]
 impl IntoRawFd for File {
     #[inline]
@@ -239,10 +274,10 @@ impl IntoRawFd for File {
 }
 
 #[cfg(not(windows))]
-impl IntoFd for File {
+impl From<File> for OwnedFd {
     #[inline]
-    fn into_fd(self) -> OwnedFd {
-        self.std.into_fd()
+    fn from(file: File) -> OwnedFd {
+        file.std.into()
     }
 }
 
@@ -255,10 +290,10 @@ impl IntoRawHandle for File {
 }
 
 #[cfg(windows)]
-impl IntoHandle for File {
+impl From<File> for OwnedHandle {
     #[inline]
-    fn into_handle(self) -> OwnedHandle {
-        self.std.into_handle()
+    fn from(file: File) -> OwnedHandle {
+        file.std.into()
     }
 }
 
@@ -270,6 +305,14 @@ impl IntoRawHandleOrSocket for File {
     }
 }
 
+#[cfg(windows)]
+impl From<File> for OwnedHandleOrSocket {
+    #[inline]
+    fn from(file: File) -> Self {
+        file.std.into()
+    }
+}
+
 impl Read for File {
     #[inline]
     fn read(&mut self, buf: &mut [u8]) -> io::Result<usize> {
@@ -410,7 +453,6 @@ impl Seek for File {
         self.std.seek(pos)
     }
 
-    #[cfg(seek_convenience)]
     #[inline]
     fn stream_position(&mut self) -> io::Result<u64> {
         self.std.stream_position()
@@ -423,7 +465,6 @@ impl Seek for &File {
         (&mut &self.std).seek(pos)
     }
 
-    #[cfg(seek_convenience)]
     #[inline]
     fn stream_position(&mut self) -> io::Result<u64> {
         (&mut &self.std).stream_position()
@@ -438,58 +479,58 @@ impl From<File> for process::Stdio {
 }
 
 #[cfg(unix)]
-impl std::os::unix::fs::FileExt for File {
+impl crate::fs::FileExt for File {
     #[inline]
     fn read_at(&self, buf: &mut [u8], offset: u64) -> io::Result<usize> {
-        self.std.read_at(buf, offset)
+        std::os::unix::fs::FileExt::read_at(&self.std, buf, offset)
     }
 
     #[inline]
     fn write_at(&self, buf: &[u8], offset: u64) -> io::Result<usize> {
-        self.std.write_at(buf, offset)
+        std::os::unix::fs::FileExt::write_at(&self.std, buf, offset)
     }
 
     #[inline]
     fn read_exact_at(&self, buf: &mut [u8], offset: u64) -> io::Result<()> {
-        self.std.read_exact_at(buf, offset)
+        std::os::unix::fs::FileExt::read_exact_at(&self.std, buf, offset)
     }
 
     #[inline]
     fn write_all_at(&self, buf: &[u8], offset: u64) -> io::Result<()> {
-        self.std.write_all_at(buf, offset)
+        std::os::unix::fs::FileExt::write_all_at(&self.std, buf, offset)
     }
 }
 
 #[cfg(target_os = "wasi")]
-impl std::os::wasi::fs::FileExt for File {
+impl crate::fs::FileExt for File {
     #[inline]
     fn read_at(&self, bufs: &mut [u8], offset: u64) -> io::Result<usize> {
-        self.std.read_at(bufs, offset)
+        std::os::wasi::fs::FileExt::read_at(&self.std, bufs, offset)
     }
 
     #[inline]
     fn write_at(&self, bufs: &[u8], offset: u64) -> io::Result<usize> {
-        self.std.write_at(bufs, offset)
+        std::os::wasi::fs::FileExt::write_at(&self.std, bufs, offset)
     }
 
     #[inline]
     fn read_vectored_at(&self, bufs: &mut [IoSliceMut], offset: u64) -> io::Result<usize> {
-        self.std.read_vectored_at(bufs, offset)
+        std::os::wasi::fs::FileExt::read_vectored_at(&self.std, bufs, offset)
     }
 
     #[inline]
     fn write_vectored_at(&self, bufs: &[IoSlice], offset: u64) -> io::Result<usize> {
-        self.std.write_vectored_at(bufs, offset)
+        std::os::wasi::fs::FileExt::write_vectored_at(&self.std, bufs, offset)
     }
 
     #[inline]
-    fn tell(&self) -> std::result::Result<u64, std::io::Error> {
-        self.std.tell()
+    fn tell(&self) -> std::result::Result<u64, io::Error> {
+        std::os::wasi::fs::FileExt::tell(&self.std)
     }
 
     #[inline]
-    fn fdstat_set_flags(&self, flags: u16) -> std::result::Result<(), std::io::Error> {
-        self.std.fdstat_set_flags(flags)
+    fn fdstat_set_flags(&self, flags: u16) -> std::result::Result<(), io::Error> {
+        std::os::wasi::fs::FileExt::fdstat_set_flags(&self.std, flags)
     }
 
     #[inline]
@@ -497,31 +538,31 @@ impl std::os::wasi::fs::FileExt for File {
         &self,
         rights: u64,
         inheriting: u64,
-    ) -> std::result::Result<(), std::io::Error> {
-        self.std.fdstat_set_rights(rights, inheriting)
+    ) -> std::result::Result<(), io::Error> {
+        std::os::wasi::fs::FileExt::fdstat_set_rights(&self.std, rights, inheriting)
     }
 
     #[inline]
-    fn advise(&self, offset: u64, len: u64, advice: u8) -> std::result::Result<(), std::io::Error> {
-        self.std.advise(offset, len, advice)
+    fn advise(&self, offset: u64, len: u64, advice: u8) -> std::result::Result<(), io::Error> {
+        std::os::wasi::fs::FileExt::advise(&self.std, offset, len, advice)
     }
 
     #[inline]
-    fn allocate(&self, offset: u64, len: u64) -> std::result::Result<(), std::io::Error> {
-        self.std.allocate(offset, len)
+    fn allocate(&self, offset: u64, len: u64) -> std::result::Result<(), io::Error> {
+        std::os::wasi::fs::FileExt::allocate(&self.std, offset, len)
     }
 
     #[inline]
-    fn create_directory<P: AsRef<Path>>(&self, dir: P) -> std::result::Result<(), std::io::Error> {
-        self.std.create_directory(dir)
+    fn create_directory<P: AsRef<Path>>(&self, dir: P) -> std::result::Result<(), io::Error> {
+        std::os::wasi::fs::FileExt::create_directory(&self.std, dir)
     }
 
     #[inline]
     fn read_link<P: AsRef<Path>>(
         &self,
         path: P,
-    ) -> std::result::Result<std::path::PathBuf, std::io::Error> {
-        self.std.read_link(path)
+    ) -> std::result::Result<std::path::PathBuf, io::Error> {
+        std::os::wasi::fs::FileExt::read_link(&self.std, path)
     }
 
     #[inline]
@@ -529,31 +570,31 @@ impl std::os::wasi::fs::FileExt for File {
         &self,
         lookup_flags: u32,
         path: P,
-    ) -> std::result::Result<std::fs::Metadata, std::io::Error> {
-        self.std.metadata_at(lookup_flags, path)
+    ) -> std::result::Result<std::fs::Metadata, io::Error> {
+        std::os::wasi::fs::FileExt::metadata_at(&self.std, lookup_flags, path)
     }
 
     #[inline]
-    fn remove_file<P: AsRef<Path>>(&self, path: P) -> std::result::Result<(), std::io::Error> {
-        self.std.remove_file(path)
+    fn remove_file<P: AsRef<Path>>(&self, path: P) -> std::result::Result<(), io::Error> {
+        std::os::wasi::fs::FileExt::remove_file(&self.std, path)
     }
 
     #[inline]
-    fn remove_directory<P: AsRef<Path>>(&self, path: P) -> std::result::Result<(), std::io::Error> {
-        self.std.remove_directory(path)
+    fn remove_directory<P: AsRef<Path>>(&self, path: P) -> std::result::Result<(), io::Error> {
+        std::os::wasi::fs::FileExt::remove_directory(&self.std, path)
     }
 }
 
 #[cfg(windows)]
-impl std::os::windows::fs::FileExt for File {
+impl crate::fs::FileExt for File {
     #[inline]
     fn seek_read(&self, buf: &mut [u8], offset: u64) -> io::Result<usize> {
-        self.std.seek_read(buf, offset)
+        std::os::windows::fs::FileExt::seek_read(&self.std, buf, offset)
     }
 
     #[inline]
     fn seek_write(&self, buf: &[u8], offset: u64) -> io::Result<usize> {
-        self.std.seek_write(buf, offset)
+        std::os::windows::fs::FileExt::seek_write(&self.std, buf, offset)
     }
 }
 
